from naas_drivers.driver import InDriver, OutDriver
import requests as r
import os

AUTH_API_FQDN = "auth.naas.ai"
AUTH_API_PROTOCOL = "https"


class NaasAuth(InDriver, OutDriver):
    __access_token = None
    __headers = None

    def __init__(self):
        self.user = self.Users(self)
        self.bearer = self.Bearer(self)

    @property
    def access_token(self):
        return self.__access_token

    @property
    def headers(self):
        return self.__headers

    def connect(self, token=None):
        if not token:
            token = os.environ.get("JUPYTERHUB_API_TOKEN")
        res = r.get(
            f"{AUTH_API_PROTOCOL}://{AUTH_API_FQDN}/bearer/jupyterhubtoken",
            params={"token": token},
        )
        res.raise_for_status()
        if res.status_code == 200:
            self.__access_token = res.json().get("access_token")
            self.__headers = {"Authorization": f"Bearer {self.__access_token}"}
        return self

    class __InnerBase:
        """__InnerBase class"""

        def __init__(self, parent):
            self.__parent = parent

        @property
        def headers(self):
            return self.__parent.headers

    class Users(__InnerBase):
        """Users inner class"""

        def me(self):
            res = r.get(
                f"{AUTH_API_PROTOCOL}://{AUTH_API_FQDN}/users/me/", headers=self.headers
            )
            res.raise_for_status()
            if res.status_code == 200:
                return res.json()

    class Bearer(__InnerBase):
        """Bearer inner class"""

        def validate(self):
            res = r.get(
                f"{AUTH_API_PROTOCOL}://{AUTH_API_FQDN}/bearer/validate",
                headers=self.headers,
            )
            res.raise_for_status()
            if res.status_code == 200:
                return res.json()



from naas_drivers.driver import InDriver, OutDriver
from naas_drivers.tools.naas_auth import NaasAuth
from datetime import datetime
import requests as r

CREDITS_API_FQDN = "credits.naas.ai"
AUTH_API_PROTOCOL = "https"


class NaasCredits(InDriver, OutDriver):
    __access_token = None
    __headers = None

    def __init__(self):
        self.transactions = self.Transactions(self)

    @property
    def headers(self):
        return self.__headers

    def connect(self, token=None):
        naas_auth = NaasAuth().connect(token)
        self.__access_token = naas_auth.access_token
        self.__headers = naas_auth.headers
        return self

    def get_plan(self):
        res = r.get(
            f"{AUTH_API_PROTOCOL}://{CREDITS_API_FQDN}/plan", headers=self.headers
        )
        res.raise_for_status()
        return res.json()

    def get_balance(self):
        res = r.get(
            f"{AUTH_API_PROTOCOL}://{CREDITS_API_FQDN}/balance", headers=self.headers
        )
        res.raise_for_status()
        return res.json()

    class __InnerBase:
        """__InnerBase class"""

        def __init__(self, parent):
            self.__parent = parent

        @property
        def headers(self):
            return self.__parent.headers

    class Transactions(__InnerBase):
        def get_currents(self):
            res = r.get(
                f"{AUTH_API_PROTOCOL}://{CREDITS_API_FQDN}/transactions/current",
                headers=self.headers,
            )
            res.raise_for_status()
            return res.json()

        def get(
            self,
            page_size: int = None,
            page_number: int = None,
            start_date: datetime = None,
            end_date: datetime = None,
            order_by: str = None,
        ):
            params = {
                "page_size": page_size,
                "page_number": page_number,
                "start_date": start_date,
                "end_date": end_date,
                "order_by": order_by,
            }
            params = {k: params[k] for k in params.keys() if params[k] is not None}

            res = r.get(
                f"{AUTH_API_PROTOCOL}://{CREDITS_API_FQDN}/transactions",
                headers=self.headers,
                params=params,
            )
            res.raise_for_status()
            return res.json()

        def import_bulk(self, username, files):
            res = r.post(
                f"{AUTH_API_PROTOCOL}://{CREDITS_API_FQDN}/transactions/import/bulk",
                headers=self.headers,
                files=files,
                params={"username": username},
            )
            res.raise_for_status()
            return res.json()



import os
import requests
import plotly.express as px

config = {"displayModeBar": False}


class Plotly:
    __css_base = ".modebar {display: none;} \n.modebar-container {display: none;} "

    def __export(self, chart, filename, css=None):
        html_filename = f"{filename.split('.')[0]}.html"
        html_exist = os.path.exists(html_filename)
        chart.write_html(html_filename)
        selector = None
        html_map = None
        if css is None:
            css = self.__css_base
        else:
            css = css + self.__css_base
        with open(html_filename) as f:
            html_map = f.read()
            if html_map.find(" cartesianlayer") != -1:
                selector = ".cartesianlayer"
            result = html_map.replace(
                "</head>", f'<style id="naas_css">{css}</style></head>'
            )
        with open(html_filename, "w") as f:
            f.write(result)
            f.close()
        if filename.endswith(".png") or filename.endswith(".jpeg"):
            html_text = result
            extension = filename.split(".")[1]
            json = {
                "output": "screenshot",
                "html": html_text,
                "emulateScreenMedia": True,
                "ignoreHttpsErrors": True,
                "scrollPage": False,
                "screenshot": {"type": extension},
            }
            if selector:
                json["screenshot"]["selector"] = selector
            req = requests.post(
                url=f"{os.environ.get('SCREENSHOT_API', 'http://naas-screenshot:9000')}/api/render",  # Sensitive
                json=json,
            )
            req.raise_for_status()
            open(filename, "wb").write(req.content)
            if not html_exist:
                os.remove(html_filename)
        elif not filename.endswith(".html"):
            print("Not supported for now")
            os.remove(html_filename)
            return
        print(f"Saved as {filename}")

    def export(self, chart, filenames, css=None):
        """create html export and add css to it"""
        if isinstance(filenames, list):
            for filename in filenames:
                self.__export(chart, filename, css)
        else:
            self.__export(chart, filenames, css)

    def __update_layout(
        self,
        fig,
        title=None,
        plot_bgcolor=None,
        width=None,
        height=None,
        showlegend=None,
        xaxis_title=None,
        yaxis_title=None,
    ):
        fig.update_layout(
            title=title,
            plot_bgcolor=plot_bgcolor,
            width=width,
            height=height,
            showlegend=showlegend,
            xaxis_title=xaxis_title,
            yaxis_title=yaxis_title,
        )
        return fig

    def linechart(
        self,
        df,
        x,
        y,
        title=None,
        plot_bgcolor="#ffffff",
        width=None,
        height=600,
        showlegend=False,
        xaxis_title=None,
        yaxis_title=None,
    ):
        fig = px.line(df, x, y)
        fig = self.__update_layout(
            fig,
            title,
            plot_bgcolor,
            width,
            height,
            showlegend,
            xaxis_title,
            yaxis_title,
        )
        fig.show(config=config)
        return fig



from naas_drivers.driver import InDriver, OutDriver
from naas_drivers.tools.naas_auth import NaasAuth
import requests
from typing import List

EVENTS_API_FQDN = "events.naas.ai"


class NaasEvents(InDriver, OutDriver):
    __headers = None

    def connect(self, token=None) -> "NaasEvents":
        naas_auth = NaasAuth().connect(token)
        self.__headers = naas_auth.headers
        return self

    def add_events(self, events: List[any]):
        res = requests.post(
            f"https://{EVENTS_API_FQDN}/events", headers=self.__headers, json=events
        )
        res.raise_for_status()
        return res.json()

    def user_me(self):
        res = requests.get(f"https://{EVENTS_API_FQDN}/user/me", headers=self.__headers)
        res.raise_for_status()
        return res.json()



from typing import List
import logging
import sys
from pandas import DataFrame

import snowflake.connector
from snowflake.connector.errors import ProgrammingError
from naas_drivers.driver import InDriver, OutDriver


global snowflake_instance


class Snowflake(InDriver, OutDriver):
    """
    Driver for Snowflake Data Warehouse. Enables to connect to the SF account and execute queries.
    There is no need to log in and use Worksheets. Connection instance is enough to run any command.

    Basic drill recommendation:
        - `snowflake.connect()` - to connect to the account with given credentials
        - `snowflake.<warehouse/schema/database>()` - to eventually set up SF working environment for the whole session
        - `snowflake.execute()` - to run any command you want
        - `snowflake.query() / snowflake.query_pd()` - additional, more convenient functionality to run DQL queries,
            and, eventually, return data in form of the pandas.DataFrame.

    For more information, please refer to functions' documentation.

    Moreover, there are 2 extra session's objects accessible: `cursor` and `connection` that are raw
        `snowflake-python-connector` objects. If you need, you can refer to Snowflake docs and perform extra tasks that
        aren't (yet) implemented in this driver.

    If the functionality you're looking for is not implemented yet, please create an issue in
        (naas-drivers repository)[https://github.com/jupyter-naas/drivers/tree/main],
        we would be more than happy to help.
    """

    def __init__(self):

        self._connection = None
        self._cursor = None
        self._warehouse = None
        self._database = None
        self._schema = None
        self._role = None

        global snowflake_instance
        snowflake_instance = self

    @staticmethod
    def instance():
        return snowflake_instance

    @property
    def connection(self):
        return self._connection

    @property
    def cursor(self):
        return self._cursor

    @property
    def warehouse(self) -> str:
        return self._warehouse

    @property
    def database(self) -> str:
        return self._database

    @property
    def schema(self) -> str:
        return self._schema

    @property
    def role(self) -> str:
        return self._role

    @warehouse.setter
    def warehouse(self, warehouse: str) -> None:
        if isinstance(warehouse, str):
            self.cursor.execute(f"USE WAREHOUSE {warehouse};")
            self._warehouse = warehouse
        else:
            raise TypeError("Wrong type for property [warehouse] - `str` required")

    @database.setter
    def database(self, database: str) -> None:
        if isinstance(database, str):
            self.cursor.execute(f"USE DATABASE {database};")
            self._database = database
        else:
            raise TypeError("Wrong type for property [database] - `str` required")

    @schema.setter
    def schema(self, schema: str) -> None:
        if isinstance(schema, str):
            self.cursor.execute(f"USE SCHEMA {schema};")
            self._schema = schema
        else:
            raise TypeError("Wrong type for property [schema] - `str` required")

    @role.setter
    def role(self, role: str) -> None:
        if isinstance(role, str):
            self.cursor.execute(f"USE ROLE {role};")
            self._role = role
        else:
            raise TypeError("Wrong type for property [role] - `str` required")

    def connect(
        self,
        account: str,
        username: str,
        password: str,
        warehouse: str = "",
        database: str = "",
        schema: str = "",
        role: str = "",
    ) -> None:
        """
        Connects to Snowflake account with given credentials.
        Connection is established and both `connection` and `cursor` are being set up
        @param account: SF account identifier, can be fetched from login URL,
            e.g. <account_identifier>.snowflakecomputing.com
        @param username: SF account username
        @param password: SF account password
        @param warehouse: (optional) SF warehouse to set up while creating a connection
        @param database: (optional) SF database to set up while creating a connection
        @param schema: (optional) SF schema to set up while creating a connection
        @param role: (optional) SF role to set up while creating a connection
        """
        self._connection = snowflake.connector.connect(
            account=account, user=username, password=password
        )
        self._cursor = self._connection.cursor()
        self._set_environment(warehouse, database, schema, role)
        self.connected = True

    def execute(
        self,
        sql: str,
        warehouse: str = "",
        database: str = "",
        schema: str = "",
        role: str = "",
        n: int = 10,
    ) -> (List, List):
        """
        Execute passed command. Could be anything, starting from DQL query, and ending with DDL commands
        @param sql: command/query to execute
        @param warehouse: (optional) warehouse to use for passed command
        @param database: (optional) database to use for passed command
        @param schema: (optional) schema to use for passed command
        @param role: (optional) role to use for passed command
        @param n: (optional) query result length limit
        @return: List: (results, columns_metadata) containing query outcome
        """

        # Switching warehouse, database, schema, and role for function call purposes
        warehouse_old, database_old, schema_old, role_old = (
            self.warehouse,
            self.database,
            self.schema,
            self.role,
        )
        self._set_environment(warehouse, database, schema, role)
        self.warehouse, self.database, self.schema, self.role = (
            warehouse_old,
            database_old,
            schema_old,
            role_old,
        )

        res = self._cursor.execute(sql)
        if res.rowcount < n:
            n = res.rowcount
        if n > 0:
            return res.fetchmany(n), res.description
        if n == -1:
            return res.fetchall(), res.description

    def query(
        self,
        sql: str,
        warehouse: str = "",
        database: str = "",
        schema: str = "",
        role: str = "",
        n: int = 10,
    ) -> (List, List):
        """
        Query data and return results in the form of plain vanilla List: (results, columns_metadata)
        @param sql: query to execute
        @param warehouse: (optional) warehouse to use for passed query
        @param database: (optional) database to use for passed query
        @param schema: (optional) schema to use for passed query
        @param role: (optional) role to use for passed query
        @param n: (optional) query result length limit
        @return: List: (results, columns_metadata) containing query outcome
        """
        return self.execute(sql, warehouse, database, schema, role, n)

    def query_pd(
        self,
        sql: str,
        warehouse: str = "",
        database: str = "",
        schema: str = "",
        role: str = "",
        n: int = 10,
    ) -> DataFrame:
        """
        Query data and return results in the form of pandas.DataFrame
        @param sql: query to execute
        @param warehouse: (optional) warehouse to use for passed query
        @param database: (optional) database to use for passed query
        @param schema: (optional) schema to use for passed query
        @param role: (optional) role to use for passed query
        @param n: (optional) query result length limit
        @return: pandas.DataFrame table containing query outcome
        """
        res = self.query(sql, warehouse, database, schema, role, n)

        # TODO: Apply dtypes mapping from ResultMetadata objects
        return DataFrame(
            res[0], columns=[result_metadata.name for result_metadata in res[1]]
        )

    def close_connection(self) -> None:
        """
        Closes a connection to Snowflake account, resets all the internal parameters
        """
        self._cursor.close()
        self._connection.close()

        self._schema = None
        self._database = None
        self._cursor = None
        self._connection = None

    def _set_environment(
        self, warehouse: str = "", database: str = "", schema: str = "", role: str = ""
    ) -> None:
        """
        Tries to set Snowflake environment in bulk: warehouse, database, schema, and role
        @param warehouse: warehouse to set in the environment
        @param database: database to set in the environment
        @param schema: schema to set in the environment
        @param role: role to set in the environment
        """
        warehouse, database, schema, role = (
            warehouse.strip(),
            database.strip(),
            schema.strip(),
            role.strip(),
        )
        try:
            if warehouse != "":
                self.warehouse = warehouse
            if database != "":
                self.database = database
            if schema != "":
                self.schema = schema
            if role != "":
                self.role = role
        except ProgrammingError as pe:
            logging.error(f"Error while setting SF environment. More on that: {pe}")
            sys.exit()



from naas_drivers.driver import InDriver, OutDriver
import pandas as pd
import requests
from datetime import datetime
import os
import string
import json
import re

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"


class HSCRUD:
    # class HSCRUD(CRUD):
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

        # Manage message
        message_dict = {"contacts": "Contact", "company": "Company", "deals": "Deal"}
        self.msg = message_dict[self.model_name]

    def __values_format(self, data):
        for key, value in data["properties"].items():
            # Check if value is not None
            if value is not None:
                # Force format to string  & delete space
                value = str(value).strip()
                # Specific rules
                if key == "firstname":
                    value = value.replace("-", " ")
                    value = string.capwords(value).replace(" ", "-")
                elif key == "lastname":
                    value = value.upper()
                elif key == "phone":
                    if value == "nan":
                        value = ""
                    else:
                        value = value.replace(" ", "").replace(".", "")
                elif key == "closedate":
                    try:
                        value = datetime.strptime(value, "%Y-%m-%d")
                        value = str(int(value.timestamp())) + "000"
                    except ValueError:
                        print(
                            f"❌ Close date '{value}' is in wrong format.\n"
                            "Please change it to %Y-%m-%d."
                        )
                elif key == "amount":
                    if value == "nan":
                        value = 0
                    else:
                        value = value.replace(".", "")
                # Change value in dict
                data["properties"][key] = value
        return data

    def __get_by_page(self, params):
        res = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            params=params,
            allow_redirects=False,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        return res.json()

    def get_all(self, columns=None):
        items = []
        params = self.params
        if columns is not None:
            params["properties"] = columns
        more_page = True
        while more_page:
            data = self.__get_by_page(params)
            for row in data.get("results"):
                properties = row["properties"]
                items.append(properties)
            if "paging" in data:
                params["after"] = data.get("paging").get("next").get("after")
            else:
                more_page = False
        df = pd.DataFrame(items).reset_index(drop=True)
        params.pop("after")
        if columns is not None:
            params.pop("properties")
            self.params = params
            for col in columns:
                if col not in df.columns:
                    index = columns.index(col)
                    columns.pop(index)
            # Reorder columns
            df = df[columns]
        return df

    def get(self, uid, columns=None):
        params = self.params
        if columns is not None:
            params["properties"] = columns
        res = requests.get(
            url=f"{self.base_url}/{uid}",
            headers=self.req_headers,
            params=params,
            allow_redirects=False,
        )
        if columns is not None:
            params.pop("properties")
            self.params = params
        self.params = params
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        return res.json()

    def patch(self, uid, data):
        data = self.__values_format(data)
        res = requests.patch(
            url=f"{self.base_url}/{uid}",
            headers=self.req_headers,
            params=self.params,
            json=data,
            allow_redirects=False,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        # Message success
        print(f"✔️ {self.msg} (id={uid}) successfully updated.")
        return res.json()

    def send(self, data):
        data = self.__values_format(data)
        res = requests.post(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            params=self.params,
            json=data,
            allow_redirects=False,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        res_json = res.json()
        uid = res_json.get("id")
        # Message success
        print(f"✔️ {self.msg} (id={uid}) successfully created.")
        return uid

    def delete(self, uid):
        result = self.get(uid)
        if result is not None:
            res = requests.delete(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                params=self.params,
                allow_redirects=False,
            )
            try:
                res.raise_for_status()
            except requests.HTTPError as e:
                return e
            # Message success
            print(f"✔️ {self.msg} (id={uid}) successfully deleted.")
            return uid
        else:
            # Message success
            print(f"❌ {self.msg} (id={uid}) does not exist.")


class Contact(HSCRUD):
    def create(
        self,
        email,
        firstname=None,
        lastname=None,
        phone=None,
        jobtitle=None,
        website=None,
        company=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "email": email,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "jobtitle": jobtitle,
                "website": website,
                "company": company,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.send(data)
        return res

    def update(
        self,
        uid,
        email=None,
        firstname=None,
        lastname=None,
        phone=None,
        jobtitle=None,
        website=None,
        company=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "email": email,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "jobtitle": jobtitle,
                "website": website,
                "company": company,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.patch(uid, data)
        return res

    def get_id(self, email):
        uid = None
        try:
            # Try to create contact with email
            data = {"properties": {"email": email}}
            uid = self.send(data)
        except requests.HTTPError as err:
            # Get contact id in pattern if error
            pattern = r"\: (\d)+"
            response_txt = err.response.text
            uid = re.search(pattern, response_txt).group(0).replace(": ", "")
        return uid


class Deal(HSCRUD):
    def create(
        self, dealname, dealstage, closedate=None, amount=None, hubspot_owner_id=None
    ):
        data = {
            "properties": {
                "dealstage": dealstage,
                "dealname": dealname,
                "closedate": closedate,
                "amount": amount,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.send(data)
        return res

    def update(
        self,
        uid,
        dealname=None,
        dealstage=None,
        closedate=None,
        amount=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "dealstage": dealstage,
                "dealname": dealname,
                "closedate": closedate,
                "amount": amount,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.patch(uid, data)
        return res


class Pipeline:
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

    def get_all(self, pipeline=None, pipeline_id=None):
        res = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            params=self.params,
            allow_redirects=False,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        deal_stages = []
        results = res.json().get("results")
        for res in results:
            stages = res.get("stages")
            for stage in stages:
                deal_stage = {
                    "pipeline": res.get("label"),
                    "pipeline_id": res.get("id"),
                    "dealstage_label": stage.get("label"),
                    "dealstage_id": stage.get("id"),
                    "displayOrder": stage.get("displayOrder"),
                    "dealclosed": stage.get("metadata", {}).get("isClosed"),
                    "probability": stage.get("metadata", {}).get("probability"),
                    "createdAt": stage.get("createdAt"),
                    "updatedAt": stage.get("updatedAt"),
                    "archived": stage.get("archived"),
                }
                deal_stages.append(deal_stage)
        # Create DataFrame
        df = pd.DataFrame(deal_stages)
        df.createdAt = pd.to_datetime(df.createdAt).dt.strftime(DATETIME_FORMAT)
        df.updatedAt = pd.to_datetime(df.updatedAt).dt.strftime(DATETIME_FORMAT)

        # Filter on pipeline
        if pipeline is not None:
            df = df[df.pipeline == str(pipeline)]
        if pipeline_id is not None:
            df = df[df.pipeline_id == str(pipeline_id)]
        # Cleaning
        df = df.sort_values(by=["pipeline", "displayOrder"])
        df = df.reset_index(drop=True)
        return df


class Association:
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

    def get(self, object_name, object_id, associate):
        objects = ["deal"]
        object_check = True
        if object_name not in objects:
            object_check = False
            print(
                f"❌ Object '{object_name}' does not exist.\n"
                f"Please chose one in following list: {objects}"
            )
        associates = ["contact", "engagements"]
        associate_check = True
        if associate not in associates:
            associate_check = False
            print(
                f"❌ Associate '{associate}' does not exist.\n"
                f"Please chose one in following list: {associates}"
            )
        if object_check and associate_check:
            res = requests.get(
                url=f"{self.base_url}/{object_name}/{object_id}/"
                f"associations/{associate}",
                headers=self.req_headers,
                params=self.params,
                allow_redirects=False,
            )
            try:
                res.raise_for_status()
            except requests.HTTPError as e:
                return e
            data = res.json()
            df = pd.DataFrame.from_records(data["results"])
            if len(df) == 0:
                print(f"❌ Object id='{object_id}' does not have associates.")
            return df

    def create(self, object_name, object_id, associate, id_associate):
        objects = ["deal"]
        object_check = True
        if object_name not in objects:
            object_check = False
            print(
                f"❌ Object '{object_name}' does not exist.\n"
                f"Please chose one in following list: {objects}"
            )
        associates = ["contact"]
        associate_check = True
        if associate not in associates:
            associate_check = False
            print(
                f"❌ Associate '{associate}' does not exist.\n"
                f"Please chose one in following list: {associates}"
            )
        if object_check and associate_check:
            res = requests.put(
                url=f"{self.base_url}/{object_name}/{object_id}/associations/"
                f"{associate}/{id_associate}/{object_name}_to_{associate}",
                headers=self.req_headers,
                params=self.params,
                allow_redirects=False,
            )
            try:
                res.raise_for_status()
            except requests.HTTPError as e:
                return e
            print(
                f"✔️ {object_name} '{object_id}' and {associate} "
                f"'{id_associate}' successfully associated !"
            )
            return id_associate


class Note:
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url

    def create(
        self, content, contactids=None, dealids=None, companyids=None, ownerid=None
    ):
        payload = json.dumps(
            {
                "engagement": {
                    "active": "true",
                    "ownerId": ownerid,
                    "type": "NOTE",
                    "timestamp": str(int(datetime.now().timestamp())) + "000",
                },
                "associations": {
                    "contactIds": [contactids],
                    "dealIds": [dealids],
                    "companyIds": [companyids],
                },
                "attachments": [{}],
                "metadata": {"body": content},
            }
        )
        res = requests.post(
            self.base_url,
            data=payload,
            headers=self.req_headers,
            params=self.params,
            allow_redirects=False,
        )
        try:
            res.raise_for_status()
        except requests.HTTPError as e:
            return e
        # Message success
        return "✔️ Note successfully created."


class Hubspot(InDriver, OutDriver):

    base_url = os.environ.get("HUBSPOT_API_URL", "https://api.hubapi.com/crm/v3")
    api_token = None

    def connect(self, api_token):
        # Init Hubspot attribute
        self.token = api_token
        self.req_headers = {
            "accept": "application/json",
            "content-type": "application/json",
        }
        self.params = {"limit": "100", "archived": "false", "hapikey": api_token}

        # Init end point
        self.obj_url = f"{self.base_url}/objects"
        self.pip_url = f"{self.base_url}/pipelines"
        self.contacts = Contact(
            f"{self.obj_url}/contacts", self.req_headers, self.params
        )
        self.company = HSCRUD(f"{self.obj_url}/company", self.req_headers, self.params)
        self.deals = Deal(f"{self.obj_url}/deals", self.req_headers, self.params)
        self.pipelines = Pipeline(
            f"{self.pip_url}/deals", self.req_headers, self.params
        )
        self.associations = Association(
            f"{self.obj_url}", self.req_headers, self.params
        )
        self.notes = Note(
            "https://api.hubapi.com/engagements/v1/engagements",
            self.req_headers,
            self.params,
        )

        # Set connexion to active
        self.connected = True
        return self



import requests
import os
from mprop import mproperty
from subprocess import Popen, PIPE
import sys

__version__ = "0.103.0"

__github_repo = "jupyter-naas/drivers"

__doc_url = "https://naas.gitbook.io/drivers/"

__loaded_drivers = {}

if os.environ.get("NAAS_DRIVER_LIGHT_INIT"):
    exit()


def doc():
    return __doc_url


def version():
    print(__version__)


def get_last_version():
    url = f"https://api.github.com/repos/{__github_repo}/tags"
    response = requests.get(url, headers={"Accept": "application/vnd.github.v3+json"})
    return response.json()[0]["name"]


def up_to_date():
    return get_last_version() == version()


def load_driver(_func=None, *, extra_requires=""):
    def fn_wrapper(loader_fn):
        @mproperty
        def wrapper(mod):

            name = loader_fn.__name__

            if name not in __loaded_drivers:
                try:
                    loaded = loader_fn()
                    __loaded_drivers[name] = loaded
                except Exception as e:
                    if extra_requires != "":
                        naas_drivers_path = "/".join(__file__.split("/")[:-2])
                        if (
                            os.path.isfile(os.path.join(naas_drivers_path, "setup.py"))
                            is False
                        ):
                            naas_drivers_path = "naas-drivers"
                        cmd = [
                            "pip",
                            "install",
                            "--user",
                            f"{naas_drivers_path}[{extra_requires}]",
                        ]
                        print(
                            f"""
        👉 Running this command automatically to install missing requirements. $> {(" ").join(cmd)}

        ⚠️ You may need to restart your kernel / execution to be able to use the installed packages.

        💡 You can also run this command prior to execution next time to install these packages the way you want (venv, etc).
            """
                        )
                        process = Popen(cmd, stdout=PIPE, stderr=PIPE)
                        stdout, stderr = process.communicate()
                        print(stdout.decode("utf-8"))
                        print(stderr.decode("utf-8"), file=sys.stderr)
                        loaded = loader_fn()
                        __loaded_drivers[name] = loaded
                    else:
                        raise e
            return __loaded_drivers[name]

        return wrapper

    if _func is None:
        return fn_wrapper
    else:
        return fn_wrapper(_func)


@load_driver
def optimise():
    from naas_drivers.tools.optimise import Optimise

    return Optimise()


@load_driver
def cityfalcon():
    from naas_drivers.tools.cityfalcon import Cityfalcon

    return Cityfalcon()


@load_driver(extra_requires="geolocator")
def geolocator():
    from naas_drivers.tools.geolocator import Geolocator

    return Geolocator()


@load_driver(extra_requires="newsapi")
def newsapi():
    from naas_drivers.tools.newsapi import Newsapi

    return Newsapi()


@load_driver(extra_requires="cython,prediction")
def prediction():
    from naas_drivers.tools.prediction import Prediction

    return Prediction()


@load_driver(extra_requires="sentiment")
def sentiment():
    from naas_drivers.tools.sentiment import Sentiment

    return Sentiment()


@load_driver
def yahoofinance():
    from naas_drivers.tools.yahoofinance import Yahoofinance

    return Yahoofinance()


@load_driver
def pdf():
    from naas_drivers.tools.pdf import Pdf

    return Pdf()


@load_driver(extra_requires="plotly")
def plotly():
    from naas_drivers.tools.plotly import Plotly

    return Plotly()


@load_driver(extra_requires="ipython,emailbuilder")
def emailbuilder():
    from naas_drivers.tools.emailbuilder import EmailBuilder

    return EmailBuilder()


@load_driver(extra_requires="ipython,emailbuilder")
def emailBuilder():
    from naas_drivers.tools.emailbuilder import EmailBuilder

    return EmailBuilder(deprecated=True)


@load_driver(extra_requires="ipython")
def templates():
    from naas_drivers.tools.awesomenotebook import AwesomeNotebooks

    return AwesomeNotebooks()


@load_driver
def html():
    from naas_drivers.tools.emailbuilder import EmailBuilder

    return EmailBuilder(deprecated=True)


@load_driver(extra_requires="markdown")
def markdown():
    from naas_drivers.tools.markdown import Markdown

    return Markdown()


@load_driver(extra_requires="teams")
def teams():
    from naas_drivers.tools.teams import Teams

    return Teams()


@load_driver(extra_requires="slack")
def slack():
    from naas_drivers.tools.slack import Slack

    return Slack()


@load_driver
def qonto():
    from naas_drivers.tools.qonto import Qonto

    return Qonto()


@load_driver
def naasauth():
    from naas_drivers.tools.naas_auth import NaasAuth

    return NaasAuth()


@load_driver
def bubble():
    from naas_drivers.tools.bubble import Bubble

    return Bubble()


@load_driver(extra_requires="email")
def email():
    from naas_drivers.tools.email import Email

    return Email()


@load_driver
def healthcheck():
    from naas_drivers.tools.healthcheck import Healthcheck

    return Healthcheck()


@load_driver
def ifttt():
    from naas_drivers.tools.ifttt import Ifttt

    return Ifttt()


@load_driver
def integromat():
    from naas_drivers.tools.integromat import Integromat

    return Integromat()


@load_driver
def zapier():
    from naas_drivers.tools.zapier import Zapier

    return Zapier()


@load_driver(extra_requires="streamlit")
def streamlit():
    from naas_drivers.tools.streamlit import Streamlit

    return Streamlit()


@load_driver
def bobapp():
    from naas_drivers.tools.bobapp import Bobapp

    return Bobapp()


@load_driver(extra_requires="airtable")
def airtable():
    from naas_drivers.tools.airtable import Airtable

    return Airtable()


@load_driver
def jupyter():
    from naas_drivers.tools.jupyter import Jupyter

    return Jupyter()


@load_driver(extra_requires="ftp")
def ftp():
    from naas_drivers.tools.ftp import Ftp

    return Ftp()


@load_driver(extra_requires="git")
def git():
    from naas_drivers.tools.git import Git

    return Git()


@load_driver
def gsheet():
    from naas_drivers.tools.gsheet import Gsheet

    return Gsheet()


@load_driver(extra_requires="notion")
def notion():
    from naas_drivers.tools.notion import Notion

    return Notion()


@load_driver(extra_requires="mongo")
def mongo():
    from naas_drivers.tools.mongo import Mongo

    return Mongo()


@load_driver(extra_requires="ipython,toucan")
def toucan():
    from naas_drivers.tools.toucan import Toucan

    return Toucan()


@load_driver
def linkedin():
    from naas_drivers.tools.linkedin import LinkedIn

    return LinkedIn()


@load_driver
def hubspot():
    from naas_drivers.tools.hubspot import Hubspot

    return Hubspot()


@load_driver
def thinkific():
    from naas_drivers.tools.thinkific import Thinkific

    return Thinkific()


@load_driver
def naascredits():
    from naas_drivers.tools.naas_credits import NaasCredits

    return NaasCredits()


@load_driver()
def naasevents():
    from naas_drivers.tools.naas_events import NaasEvents

    return NaasEvents()


@load_driver
def budgetinsight():
    from naas_drivers.tools.budgetinsight import BudgetInsight

    return BudgetInsight()


@load_driver
def taggun():
    from naas_drivers.tools.taggun import Taggun

    return Taggun()


@load_driver(extra_requires="youtube,ml,pydash")
def youtube():
    from naas_drivers.tools.youtube import Youtube

    return Youtube()


@load_driver(extra_requires="google")
def googleanalytics():
    from naas_drivers.tools.googleanalytics import GoogleAnalytics

    return GoogleAnalytics()


@load_driver(extra_requires="pydash")
def github():
    from naas_drivers.tools.github import Github

    return Github()


@load_driver
def bazimo():
    from naas_drivers.tools.bazimo import Bazimo

    return Bazimo()


@load_driver(extra_requires="sharepoint")
def sharepoint():
    from naas_drivers.tools.sharepoint import Sharepoint

    return Sharepoint()


@load_driver(extra_requires="ml")
def huggingface():
    from naas_drivers.tools.huggingface import Huggingface

    return Huggingface()


@load_driver(extra_requires="snowflake")
def snowflake():
    from naas_drivers.tools.snowflake import Snowflake

    return Snowflake()
